# Python imports
import json

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

# Application imports



class LogList(Gtk.ListBox):
    def __init__(self):
        super(LogList, self).__init__()

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        ctx = self.get_style_context()
        ctx.add_class("log-list-listbox")

        self.set_placeholder( Gtk.Label(label="Empty Log List...") )
        self.set_selection_mode(Gtk.SelectionMode.NONE)

    def _setup_signals(self):
        def tggl_row_view(lb, row):
            frame    = row.get_child()
            revealer = frame.get_children()[0]
            revealer.set_reveal_child(not revealer.get_reveal_child())

            del frame
            del revealer

        self.connect("row-activated", tggl_row_view)

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        ...

    def add_log_entry(self, m_type: str, data: str or dict):
        message   = None
        json_data = None

        if isinstance(data, str):
            parts      = data.split("\n", 2)
            jsonStr    = parts[2]
            json_data  =  json.loads(jsonStr)
            messageStr = json.dumps(json_data, separators = (',', ':'), indent = 4)
            message    = f"{parts[0]}\r\n\r\n{messageStr}"

        if isinstance(data, dict):
            json_data = data
            message   = json.dumps(data, separators = (',', ':'), indent = 4)

        frame    = Gtk.Frame()
        revealer = Gtk.Revealer()
        label    = Gtk.Label(label=message)

        label.set_xalign(0.010)
        frame.set_label( f"> {m_type}: {json_data['id']}" if "id" in json_data.keys() else f"> {m_type}" )
        frame.set_label_align(0.010, 0.5)
        revealer.set_reveal_child(False)

        revealer.add(label)
        frame.add(revealer)
        self.add(frame)

        frame.show_all()

        del message
        del json_data
